"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getUniqueAggPointQuery = exports.getTotalCount = exports.getUniqueApplicationSessions = exports.getMetricGroupPointQuery = exports.getTopMetricGroup = exports.getUniqueFingerrintsByNoneQuery = exports.getTriageStateQuery = exports.getDistributionQuery = exports.getGroupedMetricGroupPointQuery = exports.getGroupedUniqueAggPointQuery = exports.getCompareNewFingerprintsQuery = exports.getFirstSeenFilter = exports.getCompareGroupsBaseObjectsQuery = exports.getUniqueAttributeQuery = exports.uniqueAggPointQuery = exports.totalCount = exports.uniqueApplicationSessions = exports.metricGroupPointQuery = exports.topMetricGroup = exports.uniqueFingerprintsByNone = exports.triageStateQuery = void 0;
const attributeAggregations_1 = require("../model/query/attributeAggregations");
const sort_1 = require("../model/query/sort");
exports.triageStateQuery = Object.freeze({
    group: ['fingerprint;issues;state'],
    fold: { fingerprint: [[attributeAggregations_1.AggregationOp.UNIQUE]] },
    order: [{ name: ';count', ordering: sort_1.Ordering.DESCENDING }],
    offset: 0,
    limit: 4,
});
exports.uniqueFingerprintsByNone = Object.freeze({
    group: ['*'],
    fold: { fingerprint: [[attributeAggregations_1.AggregationOp.UNIQUE]] },
    order: [{ name: ';count', ordering: sort_1.Ordering.DESCENDING }],
    offset: 0,
    limit: 1,
});
exports.topMetricGroup = Object.freeze({
    group: ['_metric_group'],
    fold: { _value: [[attributeAggregations_1.AggregationOp.SUM]] },
    order: [{ name: '_value;0', ordering: sort_1.Ordering.DESCENDING }],
    offset: 0,
    limit: 1,
    table: 'metrics',
});
exports.metricGroupPointQuery = Object.freeze({
    group: ['*'],
    fold: { _value: [[attributeAggregations_1.AggregationOp.SUM]] },
    order: [{ name: ';count', ordering: sort_1.Ordering.DESCENDING }],
    offset: 0,
    limit: 1,
    table: 'metrics',
});
exports.uniqueApplicationSessions = Object.freeze({
    group: ['*'],
    fold: { 'application.session': [[attributeAggregations_1.AggregationOp.UNIQUE]] },
    order: [{ name: ';count', ordering: sort_1.Ordering.DESCENDING }],
    offset: 0,
    limit: 1,
});
exports.totalCount = Object.freeze({
    group: ['*'],
    order: [{ name: ';count', ordering: sort_1.Ordering.DESCENDING }],
    offset: 0,
    limit: 1,
});
exports.uniqueAggPointQuery = Object.freeze({
    group: ['*'],
    fold: { _count: [[attributeAggregations_1.AggregationOp.SUM]] },
    order: [{ name: ';count', ordering: sort_1.Ordering.DESCENDING }],
    offset: 0,
    limit: 1,
});
function getQueryCopy(query) {
    return function () {
        return Object.assign({}, query);
    };
}
function getUniqueAttributeQuery(attribute) {
    return {
        group: ['*'],
        fold: { [attribute]: [[attributeAggregations_1.AggregationOp.UNIQUE]] },
        order: [{ name: ';count', ordering: sort_1.Ordering.DESCENDING }],
        offset: 0,
        limit: 1,
    };
}
exports.getUniqueAttributeQuery = getUniqueAttributeQuery;
function getCompareGroupsBaseObjectsQuery(compareAttribute, filters, hasApplicationSession, uniqueAttribute) {
    const newFilters = [Object.assign({}, filters[0])];
    const query = {
        filter: newFilters,
        group: [compareAttribute],
        fold: {
            fingerprint: [[attributeAggregations_1.AggregationOp.UNIQUE]],
            'fingerprint;issues;state': [[attributeAggregations_1.AggregationOp.DISTRIBUTION, 4]],
        },
        order: [{ name: ';group', ordering: sort_1.Ordering.DESCENDING }],
        offset: 0,
        limit: 4,
    };
    if (hasApplicationSession && query.fold) {
        query.fold['application.session'] = [[attributeAggregations_1.AggregationOp.UNIQUE]];
    }
    if (uniqueAttribute && query.fold) {
        query.fold[uniqueAttribute] = [[attributeAggregations_1.AggregationOp.UNIQUE]];
    }
    return query;
}
exports.getCompareGroupsBaseObjectsQuery = getCompareGroupsBaseObjectsQuery;
function getFirstSeenFilter(filters) {
    var _a;
    const newFilters = [Object.assign({}, filters[0])];
    const timestampRange = (_a = newFilters[0]) === null || _a === void 0 ? void 0 : _a.timestamp;
    newFilters[0]['fingerprint;first_seen'] = timestampRange;
    return newFilters;
}
exports.getFirstSeenFilter = getFirstSeenFilter;
function getCompareNewFingerprintsQuery(compareAttribute, filters) {
    return {
        filter: getFirstSeenFilter(filters),
        group: [compareAttribute],
        order: [{ name: ';group', ordering: sort_1.Ordering.DESCENDING }],
        fold: { fingerprint: [[attributeAggregations_1.AggregationOp.UNIQUE]] },
        offset: 0,
        limit: 4,
    };
}
exports.getCompareNewFingerprintsQuery = getCompareNewFingerprintsQuery;
function getGroupedUniqueAggPointQuery(group) {
    const query = getQueryCopy(exports.uniqueAggPointQuery)();
    query.group = [group];
    query.limit = 20;
    query.order = [{ name: ';group', ordering: sort_1.Ordering.DESCENDING }];
    return query;
}
exports.getGroupedUniqueAggPointQuery = getGroupedUniqueAggPointQuery;
function getGroupedMetricGroupPointQuery(group) {
    const query = getQueryCopy(exports.metricGroupPointQuery)();
    query.group = [group];
    query.limit = 20;
    query.order = [{ name: ';group', ordering: sort_1.Ordering.DESCENDING }];
    return query;
}
exports.getGroupedMetricGroupPointQuery = getGroupedMetricGroupPointQuery;
function getDistributionQuery(group) {
    return {
        group: [group],
        fold: { fingerprint: [[attributeAggregations_1.AggregationOp.UNIQUE]], guid: [[attributeAggregations_1.AggregationOp.UNIQUE]] },
        order: [{ name: ';count', ordering: sort_1.Ordering.DESCENDING }],
        offset: 0,
        limit: 4,
    };
}
exports.getDistributionQuery = getDistributionQuery;
exports.getTriageStateQuery = getQueryCopy(exports.triageStateQuery);
exports.getUniqueFingerrintsByNoneQuery = getQueryCopy(exports.uniqueFingerprintsByNone);
exports.getTopMetricGroup = getQueryCopy(exports.topMetricGroup);
exports.getMetricGroupPointQuery = getQueryCopy(exports.metricGroupPointQuery);
exports.getUniqueApplicationSessions = getQueryCopy(exports.uniqueApplicationSessions);
exports.getTotalCount = getQueryCopy(exports.totalCount);
exports.getUniqueAggPointQuery = getQueryCopy(exports.uniqueAggPointQuery);
//# sourceMappingURL=queryTemplates.js.map