"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.execQuery = exports.composeDistributionQuery = exports.composeBaseQueryBody = exports.parseQuery = exports.parseQueryConfigs = exports.addTimestampRangeToBin = void 0;
const util_1 = require("./util");
const coronerdUtils_1 = require("./coronerdUtils");
const crdbResponse_1 = require("./crdbResponse");
const queryTemplates_1 = require("./queryTemplates");
const summedEvents_1 = require("./summedEvents");
const uniqueAggregations_1 = require("./uniqueAggregations");
const index_1 = require("../model/query/index");
const filter_1 = require("../model/query/filter");
const attributeAggregations_1 = require("../model/query/attributeAggregations");
const queryItem_1 = require("../model/queryItem");
function addTimestampRangeToBin(query, attribute, filters) {
    var _a, _c, _d, _e, _f;
    const bin = (_c = (_a = query === null || query === void 0 ? void 0 : query.fold) === null || _a === void 0 ? void 0 : _a[attribute]) === null || _c === void 0 ? void 0 : _c.find((t) => t[0] === attributeAggregations_1.AggregationOp.BIN);
    if (bin && bin.length === 2) {
        const timestampRange = (_d = filters[0]) === null || _d === void 0 ? void 0 : _d.timestamp;
        const rangeStart = (_e = timestampRange === null || timestampRange === void 0 ? void 0 : timestampRange.find((a) => a[0] === filter_1.Operater.AT_LEAST)) === null || _e === void 0 ? void 0 : _e[1];
        const rangeEnd = (_f = timestampRange === null || timestampRange === void 0 ? void 0 : timestampRange.find((a) => a[0] === filter_1.Operater.AT_MOST)) === null || _f === void 0 ? void 0 : _f[1];
        if (rangeStart !== undefined && rangeEnd !== undefined) {
            const newTimestampBinAgg = [attributeAggregations_1.AggregationOp.BIN, bin[1], rangeStart, rangeEnd];
            index_1.removeAggregation(query, attribute, attributeAggregations_1.AggregationOp.BIN);
            index_1.addAggregation(query, attribute, newTimestampBinAgg);
        }
    }
}
exports.addTimestampRangeToBin = addTimestampRangeToBin;
function getOverrideAttributeToken(attribute) {
    return `<%${attribute.trim()}%>`;
}
function parseQueryConfigs(queryConfigs, universe, project) {
    if (universe && project) {
        queryConfigs = queryConfigs.filter((c) => c.backtrace_default || (c.universe_name == universe && c.project_name == project));
    }
    queryConfigs = queryConfigs.sort((a, _b) => a.universe_name === 'backtrace_default' && a.project_name === 'backtrace_default' ? 1 : -1);
    const groupConfig = queryConfigs.find((qc) => qc.key === 'group');
    if (groupConfig === null || groupConfig === void 0 ? void 0 : groupConfig.val) {
        const group = [groupConfig.val];
        return { group };
    }
    return null;
}
exports.parseQueryConfigs = parseQueryConfigs;
function parseQuery(originalQueryJson, versionAttribute) {
    let queryJson;
    if (versionAttribute) {
        const overrideTokenTarget = getOverrideAttributeToken('versionAttribute');
        queryJson = util_1.replaceAll(originalQueryJson, versionAttribute, overrideTokenTarget);
    }
    else {
        queryJson = originalQueryJson;
    }
    return JSON.parse(queryJson);
}
exports.parseQuery = parseQuery;
function composeBaseQueryBody(originalQueryJson, filters, order, versionAttribute, group, queryConfigs) {
    const query = parseQuery(originalQueryJson, versionAttribute);
    const parsedConfigs = parseQueryConfigs(queryConfigs);
    if (parsedConfigs) {
        const { group: groupFromConfig } = parsedConfigs;
        if (groupFromConfig) {
            query.group = groupFromConfig;
        }
    }
    query.filter = filters;
    query.order = order;
    if (group && Array.isArray(group) && group.length === 1) {
        query.group = group;
    }
    addTimestampRangeToBin(query, 'timestamp', filters);
    addTimestampRangeToBin(query, 'fingerprint;first_seen', filters);
    return query;
}
exports.composeBaseQueryBody = composeBaseQueryBody;
function composeDistributionQuery(filters, versionAttribute, queryConfigs) {
    let groupForQuery;
    const parsedConfigs = parseQueryConfigs(queryConfigs);
    if (!parsedConfigs || !(parsedConfigs === null || parsedConfigs === void 0 ? void 0 : parsedConfigs.group)) {
        throw Error('No group found');
    }
    if (versionAttribute &&
        parsedConfigs.group[0] === getOverrideAttributeToken('versionAttribute')) {
        groupForQuery = versionAttribute;
    }
    else {
        groupForQuery = parsedConfigs.group[0];
    }
    const query = queryTemplates_1.getDistributionQuery(groupForQuery);
    query.filter = filters;
    addTimestampRangeToBin(query, 'timestamp', filters);
    addTimestampRangeToBin(query, 'fingerprint;first_seen', filters);
    return query;
}
exports.composeDistributionQuery = composeDistributionQuery;
async function basicQueryFetch(coronerUrl, token, project, queryItem, queryOverrides, queryConfigs) {
    const { filter, order, versionAttribute, group } = queryOverrides;
    const queryForRequest = composeBaseQueryBody(queryItem.query, filter, order, versionAttribute, group, queryConfigs);
    const crdbResp = await coronerdUtils_1.fetchCrdb(token, coronerUrl, project, JSON.stringify(queryForRequest));
    return [queryForRequest, crdbResp];
}
async function summary_v2Fetch(coronerUrl, token, project, queryItem, queryOverrides) {
    const { filter } = queryOverrides;
    const newFingerprintsQuery = queryTemplates_1.getUniqueFingerrintsByNoneQuery();
    const triageQuery = queryTemplates_1.getTriageStateQuery();
    newFingerprintsQuery.filter = queryTemplates_1.getFirstSeenFilter(filter);
    triageQuery.filter = filter;
    let newFingerprints;
    let inProgress;
    let resolved;
    try {
        const newFingerprintsCrdbResp = await coronerdUtils_1.fetchCrdb(token, coronerUrl, project, JSON.stringify(newFingerprintsQuery));
        coronerdUtils_1.parseCrdbResponseForError(newFingerprintsCrdbResp.response);
        const newFingerprintRow = crdbResponse_1.parseAggregationRows(newFingerprintsCrdbResp.response).find(function (row) {
            return row.group === '*';
        });
        newFingerprints = {
            title: 'New fingerprints opened',
            count: crdbResponse_1.unpackAggregation(newFingerprintRow, 'fingerprint', attributeAggregations_1.AggregationOp.UNIQUE, 0),
        };
    }
    catch (error) {
        newFingerprints = {
            title: 'New fingerprints opened',
            count: 0,
            error: error.message ? error.message : 'Something went wrong',
        };
    }
    try {
        const triageCrdbResp = await coronerdUtils_1.fetchCrdb(token, coronerUrl, project, JSON.stringify(triageQuery));
        coronerdUtils_1.parseCrdbResponseForError(triageCrdbResp.response);
        const inProgressRow = crdbResponse_1.parseAggregationRows(triageCrdbResp.response).find(function (row) {
            return row.group === 'in-progress';
        });
        const closedRow = crdbResponse_1.parseAggregationRows(triageCrdbResp.response).find(function (row) {
            return row.group === 'resolved';
        });
        inProgress = {
            title: 'In progress fingerprints',
            count: crdbResponse_1.unpackAggregation(inProgressRow, 'fingerprint', attributeAggregations_1.AggregationOp.UNIQUE, 0),
        };
        resolved = {
            title: 'Fingerprints closed',
            count: crdbResponse_1.unpackAggregation(closedRow, 'fingerprint', attributeAggregations_1.AggregationOp.UNIQUE, 0),
        };
    }
    catch (error) {
        inProgress = {
            title: 'In progress fingerprints',
            count: 0,
            error: error.message,
        };
        resolved = {
            title: 'Fingerprints closed',
            count: 0,
            error: error.message,
        };
    }
    return {
        response: [newFingerprints, inProgress, resolved],
        status: 200,
        statusText: 'success',
    };
}
async function fetchUniqueEvents(coronerUrl, token, projectName, filter, universe, config, versionAttribute, uniqueGroup) {
    try {
        const uniqueAggregationName = await uniqueAggregations_1.getUniqueAggregationName(coronerUrl, token, projectName, universe, config, uniqueGroup);
        const [_uniqueGroupName, totalUniques] = await uniqueAggregations_1.getUniqueEvents(coronerUrl, token, projectName, filter, uniqueAggregationName);
        const groupedUniques = await uniqueAggregations_1.getGroupedUniqueEvents(coronerUrl, token, projectName, filter, uniqueAggregationName, versionAttribute);
        return {
            totalUniques,
            groupedUniques,
            uniqueAggregationName,
        };
    }
    catch (error) {
        return {
            totalUniques: undefined,
            groupedUniques: [],
            uniqueAggregationName: 'guid',
        };
    }
}
function getUniqueEventsForGroup(row, totalUniques, groupedUniques) {
    const uniqueForGroup = groupedUniques.find(function (unique) {
        return unique.group == row.group;
    });
    if (uniqueForGroup && totalUniques) {
        return { total: totalUniques, group_events: uniqueForGroup.count };
    }
    else {
        return undefined;
    }
}
async function compareGroupsFetch(coronerUrl, token, projectName, queryItem, queryOverrides, universe, config) {
    var _a;
    const { versionAttribute, filter, uniqueGroup, summedGroup } = queryOverrides;
    if (!versionAttribute) {
        return {
            response: { error: 'missing parameter versionAttribute' },
            status: 400,
            statusText: 'failure',
        };
    }
    try {
        const describeResp = await coronerdUtils_1.fetchDescribe(coronerUrl, token, projectName);
        const hasApplicationSessions = describeResp.findIndex(function (attribute) {
            return attribute.name === 'application.session' && attribute.statistics.used;
        }) >= 0;
        const { totalUniques, groupedUniques, uniqueAggregationName } = await fetchUniqueEvents(coronerUrl, token, projectName, filter, universe, config, versionAttribute, uniqueGroup);
        const compareBaseQuery = queryTemplates_1.getCompareGroupsBaseObjectsQuery(versionAttribute, filter, hasApplicationSessions, uniqueAggregationName);
        const newFingerprintsQuery = queryTemplates_1.getCompareNewFingerprintsQuery(versionAttribute, filter);
        const baseResp = await coronerdUtils_1.fetchCrdb(token, coronerUrl, projectName, JSON.stringify(compareBaseQuery));
        const newFingerprintsResp = await coronerdUtils_1.fetchCrdb(token, coronerUrl, projectName, JSON.stringify(newFingerprintsQuery));
        let summedGroupName;
        let summedEventsGroups;
        try {
            summedEventsGroups = await summedEvents_1.getGroupedSummedEvents(coronerUrl, token, projectName, universe, filter, config, summedGroup, versionAttribute);
            summedGroupName = (_a = summedEventsGroups[0]) === null || _a === void 0 ? void 0 : _a.metricGroup;
        }
        catch (error) {
            summedEventsGroups = [];
            console.error(error);
        }
        coronerdUtils_1.parseCrdbResponseForError(baseResp);
        coronerdUtils_1.parseCrdbResponseForError(newFingerprintsResp);
        const groupRows = crdbResponse_1.parseAggregationRows(baseResp.response);
        const newFingerprintsRows = crdbResponse_1.parseAggregationRows(newFingerprintsResp.response);
        const groups = groupRows.map(function (row) {
            var _a;
            const { group, count } = row;
            const unique_fingerprints = crdbResponse_1.unpackAggregation(row, 'fingerprint', attributeAggregations_1.AggregationOp.UNIQUE, 0);
            const triage_status_distribution = crdbResponse_1.unpackAggregation(row, 'fingerprint;issues;state', attributeAggregations_1.AggregationOp.DISTRIBUTION, []);
            const newFingerPrintRow = newFingerprintsRows.find(function (r) {
                return r.group == group;
            });
            const new_fingerprints = crdbResponse_1.unpackAggregation(newFingerPrintRow, 'fingerprint', attributeAggregations_1.AggregationOp.UNIQUE, 0);
            const unique_event_errors = crdbResponse_1.unpackAggregation(row, uniqueAggregationName, attributeAggregations_1.AggregationOp.UNIQUE, 0);
            const unique_application_sessions = crdbResponse_1.unpackAggregation(row, 'application.session', attributeAggregations_1.AggregationOp.UNIQUE, 0);
            const unique_events = getUniqueEventsForGroup(row, totalUniques, groupedUniques);
            let summed_events;
            const summedEventsCount = (_a = summedEventsGroups.find(function ({ group }) {
                return row.group === group;
            })) === null || _a === void 0 ? void 0 : _a.count;
            if (summedGroupName && summedEventsCount) {
                summed_events = {
                    metric_group: summedGroupName,
                    events: summedEventsCount,
                };
            }
            return {
                group,
                count,
                unique_fingerprints,
                new_fingerprints,
                triage_status_distribution,
                unique_event_errors,
                unique_application_sessions,
                unique_events,
                summed_events,
            };
        });
        return {
            response: groups,
            status: 200,
            statusText: 'success',
        };
    }
    catch (error) {
        return {
            response: { error: error.message },
            status: 400,
            statusText: 'failure',
        };
    }
}
async function distributionQueryFetch(coronerUrl, token, project, queryOverrides, queryConfigs) {
    const { filter, versionAttribute } = queryOverrides;
    try {
        const queryForRequest = composeDistributionQuery(filter, versionAttribute, queryConfigs);
        const crdbResp = await coronerdUtils_1.fetchCrdb(token, coronerUrl, project, JSON.stringify(queryForRequest));
        return [queryForRequest, crdbResp];
    }
    catch (error) {
        return [
            undefined,
            {
                response: {
                    error: error.message,
                },
                status: 200,
                statusText: 'failure',
            },
        ];
    }
}
async function execQuery(queryItem, queryOverrides, queryConfigs, coronerUrl, token, project, universe, config) {
    var _a, _c, _d;
    switch (queryItem.variant) {
        case queryItem_1.Variant.SUMMED_EVENT_POINT: {
            const crdbResp = await summedEvents_1.summedEventsPointQueryFetch(coronerUrl, token, project, universe, queryOverrides.filter, config, (_a = queryItem === null || queryItem === void 0 ? void 0 : queryItem.overrides) === null || _a === void 0 ? void 0 : _a.inverse, queryOverrides.summedGroup);
            return [undefined, crdbResp];
        }
        case queryItem_1.Variant.SUMMED_EVENT_BIN_CHART: {
            const crdbResp = await summedEvents_1.summedEventsBinQueryFetch(coronerUrl, token, project, universe, queryOverrides.filter, config, (_c = queryItem === null || queryItem === void 0 ? void 0 : queryItem.overrides) === null || _c === void 0 ? void 0 : _c.inverse, queryOverrides.timezoneOffset, queryOverrides.summedGroup);
            return [undefined, crdbResp];
        }
        case queryItem_1.Variant.UNIQUE_EVENT_POINT: {
            const crdbResp = await uniqueAggregations_1.uniqueAggPointQueryFetch(coronerUrl, token, project, universe, queryOverrides.filter, config, (_d = queryItem === null || queryItem === void 0 ? void 0 : queryItem.overrides) === null || _d === void 0 ? void 0 : _d.inverse, queryOverrides.uniqueGroup);
            return [undefined, crdbResp];
        }
        case queryItem_1.Variant.SUMMARY_V2: {
            const response = await summary_v2Fetch(coronerUrl, token, project, queryItem, queryOverrides);
            return [undefined, response];
        }
        case queryItem_1.Variant.COMPARE_GROUP_TABLE: {
            const response = await compareGroupsFetch(coronerUrl, token, project, queryItem, queryOverrides, universe, config);
            return [undefined, response];
        }
        case queryItem_1.Variant.DISTRIBUTION: {
            return await distributionQueryFetch(coronerUrl, token, project, queryOverrides, queryConfigs);
        }
        default:
            return await basicQueryFetch(coronerUrl, token, project, queryItem, queryOverrides, queryConfigs);
    }
}
exports.execQuery = execQuery;
//# sourceMappingURL=queryUtils.js.map