"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.summedEventsBinQueryFetch = exports.summedEventsPointQueryFetch = exports.getSummedEventsBin = exports.getGroupedSummedEvents = exports.getSummedEvents = exports.composeMetricDistributionQuery = exports.composeMetricSumQuery = exports.composeMetricsTableBaseFilter = void 0;
const coronerdUtils_1 = require("./coronerdUtils");
const queryTemplates_1 = require("./queryTemplates");
const crdbResponse_1 = require("./crdbResponse");
const index_1 = require("../model/query/index");
const filter_1 = require("../model/query/filter");
const attributeAggregations_1 = require("../model/query/attributeAggregations");
const sort_1 = require("../model/query/sort");
async function composeMetricsTableBaseFilter(coronerUrl, token, project, filters) {
    var _a;
    const describeResp = await coronerdUtils_1.fetchDescribe(coronerUrl, token, project, 'metrics');
    const metricAttributes = describeResp.map(function (attribute) {
        return attribute.name;
    });
    const timestampFilters = (_a = filters[0]) === null || _a === void 0 ? void 0 : _a.timestamp;
    const newFilterObject = { timestamp: timestampFilters };
    Object.keys(filters[0]).forEach(function (filterAttribute) {
        if (metricAttributes.includes(filterAttribute)) {
            newFilterObject[filterAttribute] = filters[0][filterAttribute];
        }
    });
    return newFilterObject;
}
exports.composeMetricsTableBaseFilter = composeMetricsTableBaseFilter;
async function composeMetricSumQuery(coronerUrl, token, project, filters, metricGroup, group) {
    const newFilterObject = await composeMetricsTableBaseFilter(coronerUrl, token, project, filters);
    newFilterObject._metric_group = [[filter_1.Operater.EQUAL, metricGroup]];
    let query;
    if (group) {
        query = queryTemplates_1.getGroupedMetricGroupPointQuery(group);
    }
    else {
        query = queryTemplates_1.getMetricGroupPointQuery();
    }
    query.filter = [newFilterObject];
    return query;
}
exports.composeMetricSumQuery = composeMetricSumQuery;
async function composeMetricDistributionQuery(coronerUrl, token, project, filters) {
    const query = queryTemplates_1.getTopMetricGroup();
    const newFilterObject = await composeMetricsTableBaseFilter(coronerUrl, token, project, filters);
    query.filter = [newFilterObject];
    return query;
}
exports.composeMetricDistributionQuery = composeMetricDistributionQuery;
async function getTopMetricGroupName(coronerUrl, token, project, filters) {
    var _a, _b;
    const queryForDistributionRequest = await composeMetricDistributionQuery(coronerUrl, token, project, filters);
    const crdbDistributionResp = await coronerdUtils_1.fetchCrdb(token, coronerUrl, project, JSON.stringify(queryForDistributionRequest));
    coronerdUtils_1.parseCrdbResponseForError(crdbDistributionResp.response);
    const topMetricGroup = (_b = (_a = crdbResponse_1.parseAggregationRows(crdbDistributionResp.response)) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.group;
    return topMetricGroup;
}
async function getSummedMetricGroup(coronerUrl, token, projectName, universe, filters, config, summedGroup) {
    var _a, _b, _c, _d;
    if (summedGroup && typeof summedGroup === 'string') {
        return summedGroup;
    }
    const topMetricGroup = await getTopMetricGroupName(coronerUrl, token, projectName, filters);
    if (topMetricGroup) {
        return topMetricGroup;
    }
    const currentProjectObject = (_c = (_b = (_a = config === null || config === void 0 ? void 0 : config.universes) === null || _a === void 0 ? void 0 : _a[universe]) === null || _b === void 0 ? void 0 : _b.projects_ext) === null || _c === void 0 ? void 0 : _c.find(function (project) {
        return project.name === projectName;
    });
    const metricGroups = await coronerdUtils_1.fetchBPGGet(coronerUrl, token, coronerdUtils_1.GET_METRICS_GROUPS);
    const metricGroupForProject = metricGroups.filter(function (metricGroup) {
        return metricGroup.project === currentProjectObject.pid;
    });
    const bpgMetricGroup = (_d = metricGroupForProject === null || metricGroupForProject === void 0 ? void 0 : metricGroupForProject[0]) === null || _d === void 0 ? void 0 : _d.name;
    return bpgMetricGroup;
}
async function getObjectTableQuery(coronerUrl, token, projectName, filters) {
    const describeResp = await coronerdUtils_1.fetchDescribe(coronerUrl, token, projectName);
    const hasApplicationSessions = describeResp.findIndex(function (attribute) {
        return attribute.name === 'application.session' && attribute.statistics.used;
    }) >= 0;
    if (hasApplicationSessions) {
        const query = queryTemplates_1.getUniqueApplicationSessions();
        query.filter = filters;
        return query;
    }
    else {
        const query = queryTemplates_1.getTotalCount();
        query.filter = filters;
        return query;
    }
}
async function getUniqueSessions(coronerUrl, token, projectName, filters) {
    var _a;
    const query = await getObjectTableQuery(coronerUrl, token, projectName, filters);
    const crdbResp = await coronerdUtils_1.fetchCrdb(token, coronerUrl, projectName, JSON.stringify(query));
    coronerdUtils_1.parseCrdbResponseForError(crdbResp);
    const group = (_a = crdbResponse_1.parseAggregationRows(crdbResp.response)) === null || _a === void 0 ? void 0 : _a[0];
    const applicationSessions = crdbResponse_1.unpackAggregation(group, 'application.session', attributeAggregations_1.AggregationOp.UNIQUE, 0);
    if (applicationSessions > 0) {
        return ['application.session', applicationSessions];
    }
    else {
        return ['count', group.count];
    }
}
async function getUniqueApplicationSessionsBin(coronerUrl, token, projectName, filters, timezoneOffset = 0) {
    var _a;
    const query = await getObjectTableQuery(coronerUrl, token, projectName, filters);
    query.group = ['timestamp.groups'];
    const { granDuration, range } = index_1.addQuantizeUintVCol(query, 'timestamp.groups', timezoneOffset);
    const binsLimit = Math.round((range[1] - range[0]) / granDuration);
    query.limit = binsLimit;
    query.order = [{ name: ';group', ordering: sort_1.Ordering.ASCENDING }];
    const crdbResp = await coronerdUtils_1.fetchCrdb(token, coronerUrl, projectName, JSON.stringify(query));
    coronerdUtils_1.parseCrdbResponseForError(crdbResp);
    const summedEventsBins = crdbResponse_1.parseAggregationRows(crdbResp.response).map(function (row) {
        var _a;
        const count = crdbResponse_1.unpackAggregation(row, 'application.session', attributeAggregations_1.AggregationOp.UNIQUE, row.count);
        const rangeStart = (_a = Number.parseInt(row.group)) !== null && _a !== void 0 ? _a : 0;
        return [rangeStart, rangeStart + granDuration, count];
    });
    const groupName = ((_a = query === null || query === void 0 ? void 0 : query.fold) === null || _a === void 0 ? void 0 : _a['application.session']) ? 'application.session' : 'count';
    return [groupName, summedEventsBins, range, binsLimit];
}
async function getSummedEvents(coronerUrl, token, projectName, universe, filters, config, summedGroup) {
    var _a;
    const groupName = await getSummedMetricGroup(coronerUrl, token, projectName, universe, filters, config, summedGroup);
    if (groupName) {
        const queryForSumRequest = await composeMetricSumQuery(coronerUrl, token, projectName, filters, groupName);
        const crdbSumResp = await coronerdUtils_1.fetchCrdb(token, coronerUrl, projectName, JSON.stringify(queryForSumRequest));
        coronerdUtils_1.parseCrdbResponseForError(crdbSumResp.response);
        const groupRow = (_a = crdbResponse_1.parseAggregationRows(crdbSumResp.response)) === null || _a === void 0 ? void 0 : _a[0];
        const summedEventCount = crdbResponse_1.unpackAggregation(groupRow, '_value', attributeAggregations_1.AggregationOp.SUM, 0);
        return [groupName, summedEventCount];
    }
    else {
        throw new Error('No metric groups found');
    }
}
exports.getSummedEvents = getSummedEvents;
async function getGroupedSummedEvents(coronerUrl, token, projectName, universe, filters, config, summedGroup, group) {
    const groupName = await getSummedMetricGroup(coronerUrl, token, projectName, universe, filters, config, summedGroup);
    if (groupName) {
        const queryForSumRequest = await composeMetricSumQuery(coronerUrl, token, projectName, filters, groupName, group);
        const crdbSumResp = await coronerdUtils_1.fetchCrdb(token, coronerUrl, projectName, JSON.stringify(queryForSumRequest));
        coronerdUtils_1.parseCrdbResponseForError(crdbSumResp.response);
        const groupRows = crdbResponse_1.parseAggregationRows(crdbSumResp.response);
        return groupRows.map(function (group) {
            const summedEventCount = crdbResponse_1.unpackAggregation(group, '_value', attributeAggregations_1.AggregationOp.SUM, 0);
            return { group: group.group, metricGroup: groupName, count: summedEventCount };
        });
    }
    else {
        throw new Error('No metric groups found');
    }
}
exports.getGroupedSummedEvents = getGroupedSummedEvents;
async function getSummedEventsBin(coronerUrl, token, projectName, universe, filters, config, timezoneOffset = 0, summedGroup) {
    const groupName = await getSummedMetricGroup(coronerUrl, token, projectName, universe, filters, config, summedGroup);
    if (groupName) {
        const queryForSumRequest = await composeMetricSumQuery(coronerUrl, token, projectName, filters, groupName);
        queryForSumRequest.group = ['timestamp.groups'];
        const { granDuration, range } = index_1.addQuantizeUintVCol(queryForSumRequest, 'timestamp.groups', timezoneOffset);
        const binsLimit = Math.round((range[1] - range[0]) / granDuration);
        queryForSumRequest.limit = binsLimit;
        queryForSumRequest.order = [{ name: ';group', ordering: sort_1.Ordering.ASCENDING }];
        const crdbSumResp = await coronerdUtils_1.fetchCrdb(token, coronerUrl, projectName, JSON.stringify(queryForSumRequest));
        coronerdUtils_1.parseCrdbResponseForError(crdbSumResp.response);
        const summedEventsBins = crdbResponse_1.parseAggregationRows(crdbSumResp.response).map(function (row) {
            var _a;
            const sum = crdbResponse_1.unpackAggregation(row, '_value', attributeAggregations_1.AggregationOp.SUM, 0);
            const rangeStart = (_a = Number.parseInt(row.group)) !== null && _a !== void 0 ? _a : 0;
            return [rangeStart, rangeStart + granDuration, sum];
        });
        return [groupName, summedEventsBins, range, binsLimit];
    }
    else {
        throw new Error('No metric groups found');
    }
}
exports.getSummedEventsBin = getSummedEventsBin;
function getTitle(group) {
    switch (group.toLowerCase()) {
        case 'sessions':
            return 'Crash free sessions';
        case 'application launch':
            return 'Crash free application launches';
        case 'application launches':
            return 'Crash free application launches';
        default:
            return `Crash free ${group.toLowerCase()}`;
    }
}
async function summedEventsPointQueryFetch(coronerUrl, token, projectName, universe, filters, config, inverse = true, summedGroup) {
    var _a;
    try {
        const [numeratorGroupName, numeratorCount] = await getUniqueSessions(coronerUrl, token, projectName, filters);
        const [denominatorGroupName, denominatorCount] = await getSummedEvents(coronerUrl, token, projectName, universe, filters, config, summedGroup);
        const response = {
            title: getTitle(denominatorGroupName),
            inverse,
            denominatorGroupName,
            denominatorCount,
            numeratorGroupName,
            numeratorCount,
        };
        return {
            response,
            status: 200,
            statusText: 'success',
        };
    }
    catch (error) {
        const response = {
            title: '',
            inverse,
            denominatorGroupName: '',
            denominatorCount: 0,
            numeratorGroupName: '',
            numeratorCount: 0,
            error: (_a = error.message) !== null && _a !== void 0 ? _a : 'unknown_error',
        };
        return {
            response,
            status: 200,
            statusText: 'failure',
        };
    }
}
exports.summedEventsPointQueryFetch = summedEventsPointQueryFetch;
async function summedEventsBinQueryFetch(coronerUrl, token, projectName, universe, filters, config, inverse = true, timezoneOffset = 0, summedGroup) {
    var _a;
    try {
        const [summed_group, summed_events, summed_range, summed_bins] = await getSummedEventsBin(coronerUrl, token, projectName, universe, filters, config, timezoneOffset, summedGroup);
        const [error_group, error_events, error_range, error_bins,] = await getUniqueApplicationSessionsBin(coronerUrl, token, projectName, filters, timezoneOffset);
        if (summed_range[0] !== error_range[0] || summed_range[1] !== error_range[1]) {
            return {
                response: {
                    error: 'range mismatch',
                },
                status: 200,
                statusText: 'failure',
            };
        }
        if (summed_bins !== error_bins) {
            return {
                response: {
                    error: 'bins mismatch',
                },
                status: 200,
                statusText: 'failure',
            };
        }
        const response = {
            title: getTitle(summed_group),
            summed_group,
            summed_events,
            error_group,
            error_events,
            range: summed_range,
            bins: summed_bins,
        };
        return {
            response,
            status: 200,
            statusText: 'success',
        };
    }
    catch (error) {
        return {
            response: {
                error: (_a = error.message) !== null && _a !== void 0 ? _a : 'unknown_error',
            },
            status: 200,
            statusText: 'success',
        };
    }
}
exports.summedEventsBinQueryFetch = summedEventsBinQueryFetch;
//# sourceMappingURL=summedEvents.js.map