import * as Knex from 'knex';

export async function up(knex: Knex): Promise<void> {
  return knex.schema.createTable('query_item_configs', function (t) {
    t.increments('id').unsigned().primary().defaultTo(1);
    t.timestamps(false, true);

    t.string('universe_name');
    t.index(['universe_name']);
    t.string('project_name');
    t.index(['project_name']);

    t.integer('user_id');
    t.string('user_name');
    t.index(['user_id', 'user_name']);

    t.integer('query_item_id').notNullable();
    t.foreign('query_item_id')
      .references('id')
      .inTable('query_items')
      .onUpdate('CASCADE') // If query_item PK is changed, update FK as well.
      .onDelete('CASCADE'); // If query_item is deleted, delete query_item_config as well.
    t.boolean('backtrace_default').notNullable().defaultTo(false);
    t.index(['backtrace_default']);
    t.integer('query_item_seed_id').unique();

    t.text('key').notNullable();
    t.index(['key']);

    /*
     * Please use JSON.stringify / JSON.parse to serialize and deserialize this column
     */
    t.text('val').notNullable();
  });
}

export async function down(knex: Knex): Promise<void> {
  return knex.schema.dropTable('query_item_configs');
}
